unit EnableGroup;

interface

uses
    Windows,
    Messages,
    SysUtils,
    Classes,
    Graphics,
    Controls,
    Forms,
    Dialogs,
    DsgnIntf,
    TypInfo,
    contnrs;

type
    TNotifyChange = procedure(Target: TObject) of object;

    TCustomEnableGroup = class(TComponent)
    private
    { Private declarations }
        FallowedTypes, FForbiddenTypes: TStringList;
        FEnabled: Boolean;
        FVisible: Boolean;
        FOnEnableChange: TNotifyChange;
        FGroupStrings: TObjectList; //holds the names of all grouped components
        FNamesList: TStringList;
        FOnPropertyChange: TNotifyChange;
        procedure SetEnabled(const Value: Boolean);
        procedure SetVisible(const Value: Boolean);
        procedure SetOnEnableChange(const Value: TNotifyChange);
        procedure SetOnPropertyChange(const Value: TNotifyChange);
    protected
    { Protected declarations }
        procedure ReadStringList(Reader: TReader);
        procedure WriteStringList(writer: TWriter);
        procedure Notification(AComponent: TComponent;
            Operation: TOperation); override;
        procedure Loaded; override;
        procedure DefineProperties(Filter: TFiler); override;
        procedure SetProperty(PropName: string; Value: Variant);

    public
    { Public declarations }
        constructor Create(AOwner: TComponent); override;
        destructor Destroy; override;
        procedure GreyObject(Target: TObject);
        function IsTypeLike(Value: TComponent; TypeName: string): Boolean;
        procedure UpdateList; virtual;
        property GroupList: TObjectList read FGroupStrings write FGroupStrings;
        property Enabled: Boolean read FEnabled write SetEnabled;
        property Visible: Boolean read FVisible write SetVisible;
        property OnEnableChange: TNotifyChange read FOnEnableChange write
        SetOnEnableChange;
        property OnPropertyChange: TNotifyChange read FOnPropertyChange write
        SetOnPropertyChange;
        property AllowedTypes: TStringList read FAllowedTypes;
        property ForbiddenTypes: TStringList read FForbiddenTypes;
    end;

    TEnableGroup = class(TCustomEnableGroup)
    public
        constructor Create(AOwner: TComponent); override;
    published
        property Enabled;
        property Visible;
        property OnEnableChange;
    end;

    // TEnabledata groups TDatasetObjects
    TEnableData = class(TCustomEnableGroup)
    private
        FActive: Boolean;
        procedure SetActive(const Value: Boolean);
        procedure SetOnActiveChanged(const Value: TNotifyChange);
        function GetActiveChanged: TNotifyChange;
    protected

    public
        constructor Create(AOwner: TComponent); override;
        procedure UpdateList; override;
    published
        property Active: Boolean read FActive write SetActive;
        property OnActiveChanged: TNotifyChange read GetActiveChanged write
        SetOnActiveChanged;
    end;

procedure Register;

implementation

uses enableform;

{$R *.dcr}

procedure Register;
begin
    RegisterComponents('AJB', [TEnableGroup, TEnableData]);
    RegisterComponentEditor(TCustomEnableGroup, TGroupListEditor);
end;

{ TCustomEnableGroup }

constructor TCustomEnableGroup.Create(AOwner: TComponent);
begin
    inherited;
    FGroupStrings := TObjectList.Create;
    FGroupStrings.OwnsObjects := False;
    FVisible := True;
    FEnabled := True;
    FOnEnableChange := nil;
    FNamesList := TStringList.Create;
    FNamesList.Sorted := True;
    FAllowedTypes := TStringList.Create;
    FForbiddenTypes := TStringList.Create;
end;

procedure TCustomEnableGroup.DefineProperties(Filter: TFiler);
begin
    inherited;
    Filter.DefineProperty('GroupList', ReadStringList, WriteStringList, True);
end;

destructor TCustomEnableGroup.Destroy;
begin
    FGroupStrings.Free;
    FNamesList.Free;
    FAllowedTypes.Free;
    FForbiddenTypes.Free;
    inherited;
end;

procedure TCustomEnableGroup.GreyObject(Target: TObject);
var
    PropInfo: PPropInfo;
    myEnabled: Boolean;
begin
    PropInfo := GetPropInfo(Target, 'Enabled');
    if PropInfo <> nil then
    begin
        myEnabled := (GetOrdProp(Target, PropInfo) = 1);
        PropInfo := GetPropInfo(Target, 'Color');
        if PropInfo <> nil then
            if myEnabled then
                SetOrdProp(Target, 'Color', clWindow)
            else
                SetOrdProp(Target, 'Color', clBtnFace);
    end;
end;

function TCustomEnableGroup.IsTypeLike(Value: TComponent; TypeName: string):
Boolean;
var
    ClassRef: TClass;
begin
    ClassRef := Value.ClassType;
    while (ClassRef <> nil) and (ClassRef.ClassName <> TypeName) do
    begin
        ClassRef := ClassRef.ClassParent;
    end;
    Result := (ClassRef <> nil);
end;

procedure TCustomEnableGroup.Loaded;
var
    mylist: TObjectList;
    j, k: integer;

    procedure AddComponentsToList(comp: TComponent);
     // generates a list of all components on this form
    var
        i: integer;
    begin
        if comp.ComponentCount > 0 then
            for i := 0 to comp.ComponentCount - 1 do
                AddComponentsToList(comp.Components[i]);
        Mylist.Add(comp);
    end;

begin
    mylist := TObjectList.Create;
    myList.OwnsObjects := False;
    try
        AddComponentstoList(Owner);
        FGroupStrings.Clear;
        if FNamesList.Count = 0 then Exit;
        for j := 0 to mylist.Count - 1 do
        begin
            if FNamesList.Find(TComponent(mylist[j]).Name, k) then
                FGroupStrings.Add(mylist[j]);
        end;
    finally
        mylist.Free;
        inherited;
    end;
end;

procedure TCustomEnableGroup.Notification(AComponent: TComponent;
    Operation: TOperation);
var
    i: integer;
    res: Boolean;

begin
    inherited;
    if (AComponent = Self) then Exit;
    if Assigned(FGroupStrings) then
        if (FGroupStrings.Count > 0) and (Operation = opRemove) then
        begin
            i := 0;
            res := False;
            while (i < FGroupStrings.Count) and (res = False) do
            begin
                if (TComponent(FGroupStrings[i]) = AComponent) then
                begin
                    FGroupStrings.Delete(i);
                    res := True;
                end;
                Inc(i);
            end;
        end;

end;

procedure TCustomEnableGroup.ReadStringList(Reader: TReader);
begin
    FNamesList.Text := Reader.ReadString;
end;

procedure TCustomEnableGroup.SetEnabled(const Value: Boolean);
var
    i: integer;
    PropInfo: PPropInfo;
begin
    FEnabled := Value;
    if Assigned(FGroupStrings) then
        for i := 0 to FGroupStrings.Count - 1 do
        begin
            PropInfo := GetPropInfo(FGroupStrings[i], 'enabled');
            if PropInfo <> nil then
                if Value then
                    SetOrdProp(FGroupStrings[i], 'Enabled', 1) // true=1
                else
                    SetOrdProp(FGroupStrings[i], 'Enabled', 0);
            if Assigned(FOnEnableChange) then
                FOnEnableChange(FGroupStrings[i]);
        end;
end;

procedure TCustomEnableGroup.SetOnEnableChange(const Value: TNotifyChange);
begin
    FOnEnableChange := Value;
end;

procedure TCustomEnableGroup.SetOnPropertyChange(
    const Value: TNotifyChange);
begin
    FOnPropertyChange := Value;
end;

procedure TCustomEnableGroup.SetProperty(PropName: string; Value: Variant);
var
    i: integer;
    PropInfo: PPropInfo;
begin
    for i := 0 to FGroupStrings.Count - 1 do
    begin
        PropInfo := GetPropInfo(FGroupStrings[i], PropName);
        if PropInfo <> nil then
        begin
            case PropInfo.PropType^.Kind of
                tkInteger, tkChar:
                    SetOrdProp(FGroupStrings[i], PropName,
                        Value);
                tkEnumeration:
                    if PropInfo.PropType^.Name = 'Boolean' then
                        if (Value = True) then
                            SetOrdProp(FGroupStrings[i], PropName, 1) //true=1
                        else
                            SetOrdProp(FGroupStrings[i], PropName, 0);
                tkfloat: SetFloatProp(FGroupStrings[i], PropName, Value);
                tkString, tkLString, tkWString:
                    SetStrProp(FGroupStrings[i],
                        PropName, VartoStr(Value));
            end;
            if Assigned(FOnPropertyChange) then
                FOnPropertyChange(FGroupStrings[i]);
        end;
    end;
end;

procedure TCustomEnableGroup.SetVisible(const Value: Boolean);
var
    i: integer;
    PropInfo: PPropInfo;
begin
    FVisible := Value;
    if Assigned(FGroupStrings) then
        for i := 0 to FGroupStrings.Count - 1 do
        begin
            PropInfo := GetPropInfo(FGroupStrings[i], 'Visible');
            if PropInfo <> nil then
                if Value then
                    SetOrdProp(FGroupStrings[i], 'Visible', 1) // true=1
                else
                    SetOrdProp(FGroupStrings[i], 'Visible', 0);
        end;
end;

procedure TCustomEnableGroup.UpdateList;
begin
    SetEnabled(FEnabled);                    // update all objects to the same
end;

procedure TCustomEnableGroup.WriteStringList(writer: TWriter);
var
    i: integer;
    s: string;
begin
    FNamesList.Clear;
    for i := 0 to FGroupStrings.Count - 1 do
    begin
        s := TComponent(FGroupStrings[i]).Name;
        if s <> '' then FNamesList.Add(s);
    end;

    writer.WriteString(FNamesList.Text);
end;

{ TEnableData }

constructor TEnableData.Create(AOwner: TComponent);
begin
    inherited;
    FActive := False;
    FAllowedTypes.Add('TDataSet');
end;

function TEnableData.GetActiveChanged: TNotifyChange;
begin
    Result := inherited OnPropertyChange;
end;

procedure TEnableData.SetActive(const Value: Boolean);
begin
    FActive := Value;
    SetProperty('Active', Value);
end;

procedure TEnableData.SetOnActiveChanged(const Value: TNotifyChange);
begin
    inherited OnPropertyChange := Value;
end;

procedure TEnableData.UpdateList;
begin
    SetProperty('Active', FActive);
end;

{ TEnableGroup }

constructor TEnableGroup.Create(AOwner: TComponent);
begin
    inherited;
    FAllowedTypes.Add('TControl');
    FAllowedTypes.Add('TMenuItem');
    FForbiddenTypes.Add('TForm');
end;

end.

